from imgui_bundle import imgui, imgui_ctx
from abc import ABCMeta, abstractmethod

from sqle.utl.future import Future, Promise, is_filling, is_filled


class Terminal:
  def __init__(self):
    self._sql = ""
    self._io  = _IO()

    self._req_sql = None
    self._req_fut = None

  def gui(self) -> None:
    with imgui_ctx.begin("my first window"):
      modified, self._sql = imgui.input_text(
        "##sql",
        self._sql,
      )

      if imgui.is_item_deactivated_after_edit():
        accept_enter = (self._req_sql != self._sql) or (not is_filling(self._req_fut))
        if accept_enter:
          # TODO: abort prev future
          self._req_fut = self._io.try_exec(self._sql)

      if is_filled(self._req_fut):
        msg = self._req_fut.get()
        imgui.text(msg if msg is not None else "OK")

        # TODO: display table

# an interface the Terminal window wants for handling SQL
class IO(metaclass = ABCMeta):
  @abstractmethod
  def try_exec(self, sql: str) -> Promise[str|None]:  # TODO: return also SQL result
    pass

# an impl of IO for testing
class _IO(IO):
  def try_exec(self, sql):
    print("exec: "+sql)
    return Promise().fill(None)
